//*****************************************************************
// Questa libreria si occupa di gestire e realizzare le 
// funzioni per il trattamento e la trasformazione di un codice 
// sorgente rappresentato da un insieme di valori mnemonici 
// in un codice macchina, rappresentato da valori numerici.
// la trasformazione avviene trasformando un particolare tipo
// di codice sorgente che rappresenta la codifica in codici 
// mnemonici di programmi per il simulatore di Von Neumann 
// realizzato.
//
// Nome del progetto     : Compiler files
// Nome di questo file   : translater.cpp
// Librerie standards    : stdio.h
// Altri files associati : tplus.cpp tplus.h 
//                         vmachine.cpp vmachine.h
// 
// Creatore              : Giovanni Ceglia 
// Data ultima revisione : 22/05/2003
// Per contatti          : giovanniceglia@email.it 
//*****************************************************************

#include <stdio.h>
#include "tplus.h"
#include "vmachine.h"

#if ! defined (TRANSLATER)
#define TRANSLATER

// struttura che rappresenta e mantiene un vettore di stringhe
// opportunamente pre trattate.
struct mnemonic_code{
  string81 words[81];
  unsigned char count;
 }; 

 
// struttura astratta che serve a gestire l'oggetto programma.
struct programma{
   string81 labelname[50];
   unsigned char labelvalue[50];
   unsigned char labelcount;
   string81 labelmark[50];
   unsigned char markaddr[50];
   unsigned char markcount;
   unsigned char code[256];
   unsigned char PI;
 }; 


// sottoprogramma che si occupa di gestire il tutto.
extern int genera_programma(const string81, const string81);

// carica il codice sorgente in una struttura di tipo testo.
extern void caricacodice(const string81, testo&);

// effettua la precompilazione di una linea di testo di codice.
extern void parseline(mnemonic_code&, const string81);

// effettua una predecodifica del codice trasformando una linea
// di codice precedentemente trattata, in codice macchina, ma
// senza inserire gli indirizzi degli identificatori.
extern void decodeline(programma&, const mnemonic_code&);

// crea il file eseguibile dalla macchina simulata di Von Neumann
// e lo salva su disco.
extern void salvacodice(const string81, const programma& p);

// collega e realizza l'ultimo passaggio nella generazione del codice
// macchina, effettuando la trasformazioni degli indirizzi degli 
// identificatori.
extern void linker_prog(programma&);

#endif
